@@ -1,8 +1,8 @@
-#include "io.h"
+#include "imio.h"
 #include "datatypes.h"
 #include <stdlib.h>
 #include <stdio.h>
-
+#include <string.h>
 
 
 /*
